# -*- coding:utf-8 -*-
# @FileName : utils_func.py
# @DateTime : 2022/11/16 17:34
# @Author   : wanglb
from threading import Thread, Condition, Lock
from datetime import datetime, timedelta
from typing import Iterable
import threading
import requests
import platform
import socket
import json
import re
import os


IS_LOCAL = platform.system() == 'Windows'
thread_max = threading.BoundedSemaphore(10)
semaphore = threading.Semaphore(3)


class Event:
    def __init__(self):
        self._cond = Condition(Lock())
        self._flag = False

    def _reset_internal_locks(self):
        self._cond.__init__(Lock())

    def is_set(self):
        return self._flag

    isSet = is_set

    def set(self):
        with self._cond:
            self._flag = True
            self._cond.notify_all()

    def clear(self):
        with self._cond:
            self._flag = False

    def wait(self, timeout=None):
        with self._cond:
            signaled = self._flag
            if not signaled:
                signaled = self._cond.wait(timeout)
            return signaled


class WaitThread(Thread):
    def __init__(self, interval, func, *args):
        super().__init__()
        self.interval = interval
        self.func = func
        self.args = args
        self.finish = Event()
        self.setDaemon(True)

    def cancel(self):
        self.finish.set()

    def reset(self):
        self.finish.set()
        self.run()

    def run(self):
        self.finish.wait(self.interval)
        if not self.finish.is_set():
            self.func(*self.args)
        self.finish.set()


class StartThread(threading.Thread):
    def __init__(self, func, args=None):
        threading.Thread.__init__(self)
        self.func = func
        self.args = args

    def run(self):
        if self.args:
            self.func(self.args)
        else:
            self.func()
        thread_max.release()


def multi_thread(func, lis: list):
    thread_list = []
    for each in lis:
        thread_max.acquire()
        m = StartThread(func, each)
        m.start()
        thread_list.append(m)
    for m in thread_list:
        m.join()


def start_new_thread(func, args=(), name=None):
    task = threading.Thread(target=func, args=args, name=name)
    task.setDaemon(True)
    task.start()


def search(content, lis: list, case: bool = True, index: int = 0):
    """
    二分法查找
    :param content: 要找的内容
    :param lis: 在哪里找
    :param case: 是否精确查找
    :param index: 从第几个开始找
    :return:
    """
    lis.sort()
    mid = len(lis) // 2
    if len(lis) == 1:
        if content != lis[0]:
            if case:
                return -1
            else:
                return index
        else:
            return index
    if content == lis[mid]:
        return mid
    elif content < lis[mid]:
        return search(content, lis[:mid], case, index)
    else:
        return search(content, lis[mid:], case, index + mid)


def expand(lis, none: [None, str] = None):
    """
    多层迭代对象展开，字符串不展开
    params lis: 原始的迭代对象
    params none: 遇到None的处理方式，‘exclude’ 跳过；None 保持不变；其他字符串 替换为对应的字符串
    """
    ll = []
    for each in lis:
        if isinstance(each, Iterable) and not isinstance(each, str):
            ll.extend(expand(each))
        else:
            if each is None:
                each = none
            if each != 'exclude':
                ll.append(each)
    return ll


def time_format(r_time):
    """
    字符串的時間格式轉換為真正的日期時間格式
    :param r_time: 字符串日期时间格式
    :return:
    """
    r_time = r_time.strip()
    now_time = datetime.now()
    rr_time = ''
    dt = tm = ''
    if _dt := re.findall(r'\d+年\d+月\d+日', r_time):
        dt = datetime.strptime(_dt[0], '%Y年%m月%d日').strftime('%Y-%m-%d')
    if _tm := re.findall(r'\d+:\d+:\d+', r_time):
        tm = datetime.strptime(_tm[0], '%H:%M:%S').strftime('%H:%M:%S')
    elif _tm := re.findall(r'\d+:\d+', r_time):
        tm = datetime.strptime(_tm[0], '%H:%M').strftime('%H:%M:%S')
    if tm:
        if not dt:
            dt = datetime.today().strftime('%Y-%m-%d')
        return datetime.strptime(dt + ' ' + tm, '%Y-%m-%d %H:%M:%S')
    if '周' in r_time or '星期' in r_time:
        try:
            r_time = r_time.replace('星期', '周')
            r_time = now_time - timedelta(weeks=int(r_time.split('周')[0].strip()))
        except:
            rr_time = r_time
    elif '天' in r_time and '昨天' not in r_time:
        try:
            r_time = now_time - timedelta(days=int(r_time.split('天')[0].strip()))
        except:
            rr_time = r_time
    elif '小時' in r_time:
        try:
            r_time = now_time - timedelta(hours=int(r_time.split('小時')[0].strip()))
        except:
            rr_time = r_time
    elif '分鐘' in r_time:
        try:
            r_time = now_time - timedelta(minutes=int(r_time.split('分鐘')[0].strip()))
        except:
            rr_time = r_time
    elif '剛剛' in r_time:
        r_time = now_time
    elif '發佈' in r_time:
        li = r_time.split(' ')
        if li[0] == '昨天':
            try:
                rr_time = (datetime.now() - timedelta(days=1)).strftime('%Y/%m/%d ') + li[1]
                rr_time = datetime.strptime(rr_time, '%Y/%m/%d %H:%M')
            except:
                rr_time = r_time
    elif '月' in r_time and '日' in r_time:
        r_t = f'{now_time.year}年{r_time} 00:00'
        try:
            rr_t = re.findall(r'\d{4}年\d{1,2}月\d{1,2}日 \d{1,2}:\d{1,2}', r_t)
            rr_time = datetime.strptime(rr_t[0], '%Y年%m月%d日 %H:%M')
        except:
            rr_time = r_time
    else:
        rr_time = r_time
    if not rr_time:
        rr_time = r_time
    return rr_time


def clear_character(st):
    """
    清除无效字符串
    :param st: 原始字符串
    :return:
    """
    comp = re.compile(r'[\u4e00-\u9fa50-9a-zA-Z~!@#$%^&*()_+=\-\\|:：；‘”“’";\'\[\]{}/\n.,，。？? ！`—【】「」『』]+')
    st = ''.join(comp.findall(st))
    return st


def func_st_datetime(x):
    """
    转换为日期时间格式，一般在pandas处理数据使用
    :param x: 日期时间格式，可能会有错误值
    :return:
    """
    try:
        return x.strftime('%Y-%m-%d %H:%M:%S')
    except:
        return ''


def func_st_date(x):
    """
    转换为日期格式，一般在pandas处理数据使用
    :param x: 日期时间格式，可能会有错误值
    :return:
    """
    try:
        return x.strftime('%Y-%m-%d')
    except:
        return ''


def func_st_time(x):
    """
    转换为时间格式，一般在pandas处理数据使用
    :param x: 日期时间格式，可能会有错误值
    :return:
    """
    try:
        return x.strftime('%H:%M:%S')
    except:
        return ''


def parse_html(html):
    lis = re.findall('>([^<]+)<', html)
    return [each.strip() for each in lis if each.strip()]


def get_outer_ip():
    """
    预计耗时0.87s
    :return:
    """
    url = 'http://ip.42.pl/raw'
    try:
        r = requests.get(url)
        if r.status_code == 200:
            return r.text
    except:
        pass
    return ''


def get_outer_ip2():
    """
    预计耗时0.38s
    :return:
    """
    url = 'http://sspanel.net/ip.php'
    try:
        r = requests.get(url)
        if r.status_code == 200:
            ips = re.findall(r'((\d+\.){3}\d+)', r.text)
            if ips:
                return ips[0][0]
    except:
        pass
    return ''


def get_inner_ip():
    """
    预计耗时0.21s
    :return:
    """
    url = 'https://tools.getquicker.cn/api/Ip/MyIp'
    try:
        r = requests.get(url)
        if r.status_code == 200:
            return r.text.strip()
    except:
        pass
    return ''


def get_lan_ip():
    """
    预计耗时0.00018s
    :return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(('8.8.8.8', 80))
    return s.getsockname()[0]


# 暂时使用本地数据读取
def get_user():
    if not IS_LOCAL:
        return ''
    path = os.path.join(os.environ['LOCALAPPDATA'], 'Quicker', 'data', 'static_data.ini')
    if not os.path.exists(path):
        return ''
    try:
        with open(path, 'r', encoding='utf-8') as f:
            st = f.read()
        return json.loads(st.replace('\ufeff', '')).get('user', '')
    except:
        return ''


# 用户信息需要考虑凭证过期问题
def update_user():
    ...


def date_add(date=None, days=0, st_format='str'):
    """
    取日期值，不带时间
    :param date: 初始日期，默认当天
    :param days: 增加天数
    :param st_format: 输出格式 str: 字符串格式, 其他: 日期格式
    :return:
    """
    if not date:
        date = datetime.today()
    if isinstance(date, str):
        date = datetime.strptime(date.split(' ')[0], '%Y-%m-%d')
    result_date = date + timedelta(days=days)
    date_st = result_date.strftime('%Y-%m-%d')
    if st_format == 'str':
        return date_st
    return datetime.strptime(date_st, '%Y-%m-%d')



# -*- coding:utf-8 -*-
# @FileName :recordstate.py
# @DateTime :2022/3/31 10:25
# @Author   :wanglb
# 需要Mongodb数据库的支持
from datetime import datetime, date
import yaml
import sys
import os

try:
    from pymongo import MongoClient
except:
    pass

"""
mongo 操作符
$lt		小于
$gt		大于
$lte	小于等于
$regex	正则
$in		包含
$size	元素数量
"""


class RecordState(object):
    """
    作为状态记录的基类
    动作状态可以继承，也可以初始化写
    """
    def __init__(self, class_name=''):
        self._base_class_name = 'RecordState'
        self.class_name = class_name or self.__subclass_name()
        self._state_base = dict()
        self._state_this = dict()
        self._base_dir = ''
        self.__init_dir()
        self._this_file = os.path.join(self._base_dir, self.class_name + '.yml')
        self._base_file = os.path.join(self._base_dir, self._base_class_name + '.yml')
        self.__read_base_file()
        self.__read_this_file()

    def __init_dir(self):
        self._base_dir = self.__get_site() + r'\auto_state'
        if not os.path.exists(self._base_dir):
            os.mkdir(self._base_dir)

    def __read_this_file(self):
        if os.path.isfile(self._this_file):
            with open(self._this_file, 'r', encoding='utf-8') as f:
                self._state_this = yaml.full_load(f)

    def __read_base_file(self):
        if os.path.isfile(self._base_file):
            with open(self._base_file, 'r', encoding='utf-8') as f:
                self._state_base = yaml.full_load(f)

    def __save_this_file(self):
        with open(self._this_file, 'w+', encoding='utf-8') as f:
            yaml.dump(self._state_this, f)

    def __save_base_file(self):
        with open(self._base_file, 'w+', encoding='utf-8') as f:
            yaml.dump(self._state_base, f)

    @classmethod
    def __subclass_name(cls):
        # 返回子类的类名
        return cls.__name__

    @staticmethod
    def __get_site():
        frame = sys._getframe()
        filename = frame.f_code.co_filename
        return os.path.dirname(filename)

    def read_state(self, key, default):
        """
        读取动作状态
        :param key: 变量名
        :param default: 不存在时，默认值
        :return: 返回对应变量名的结果
        """
        return self._state_this.get(key, default)

    def write_state(self, key, value):
        """
        写入动作状态
        :param key: 变量名
        :param value: 变量内容
        :return: None
        """
        self._state_this[key] = value
        self.__save_this_file()

    def read_base_state(self, key, default):
        """
        读取全局状态
        :param key: 变量名
        :param default: 不存在时，默认值
        :return: 返回对应变量名的结果
        """
        return self._state_base.get(key, default)

    def write_base_state(self, key, value):
        """
        写入全局状态
        :param key: 变量名
        :param value: 不存在时，默认值
        :return: None
        """
        self._state_base[key] = value
        self.__save_base_file()


class LogRecord(object):
    """
    程序运行日志记录部分，记录在mongodb数据库中
    """
    def __init__(self, collect_name=''):
        collect_name = collect_name or 'other'
        client = MongoClient('mongodb://localhost:27017/')
        db = client['local_log_record']
        self.__collect = db[collect_name]

    @staticmethod
    def __to_log(msg, time_show=True, end='\n'):
        if time_show:
            current_time = datetime.now().strftime('%m-%d %H:%M:%S')
            msg_in = f'{current_time} -> {msg}'
        else:
            msg_in = f'{msg}'
        print(msg_in, end=end)

    def record(self, dic):
        if '_record_time' not in dic:
            _record_time = datetime.now()
            dic['_record_time'] = _record_time
        dic['run_date'] = datetime.strptime(str(date.today()), '%Y-%m-%d')
        try:
            self.__collect.insert_one(dic)
        except Exception as e:
            self.__to_log(f'日志记录出错\n{e}')

    def query_all(self, arg=None, project=None):
        if not project:
            project = {}
        if '_id' not in project:
            project['_id'] = 0
        if arg and isinstance(arg, dict):
            data = list(self.__collect.find(arg, projection=project))
        else:
            data = list(self.__collect.find(projection=project))
        return data


class LogOutput(object):
    def __init__(self, module='Test'):
        try:
            self.record = LogRecord(module)
        except:
            self.record = None

    @staticmethod
    def to_log(msg, time_show=True, end='\n', ):
        if time_show:
            dt = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            msg = f'{dt} -> {msg}'
        print(msg, end=end)

    def record_log(self, txt, body=None, _type='info'):
        if not self.record:
            self.to_log('没有MongoDB，无法记录相关日志')
            return
        if not body:
            body = ''
        self.record.record({'content': {'text': txt, 'body': body}, 'type': _type})



from setuptools import setup, find_packages

setup(
    name='wanglb_utils',  # 包名
    version='0.0.4',  # 版本
    description='常用函数方法汇总',  # 包简介
    long_description=open('README.md').read(),  # 读取文件中介绍包的详细内容
    include_package_data=True,  # 是否允许上传资源文件
    author='wanglb',  # 作者
    author_email='',  # 作者邮件
    maintainer='',  # 维护者
    maintainer_email='',  # 维护者邮件
    license='MIT License',  # 协议
    url='https://pypi.org/project/wanglb_utils/',  # github或者自己的网站地址
    packages=[
        'wanglb_utils'
    ],  # 包的目录
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',  # 设置编写时的python版本
    ],
    python_requires='>=3.7',  # 设置python版本要求
    install_requires=[
        'requests >= 2.0.0',
        'yaml>=6.0'
    ],  # 安装所需要的库
    entry_points={
        'console_scripts': [
            ''],
    },
)
